//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2019 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package main

import (
	"net"
	"net/http"
	"time"

	"github.com/gorilla/mux"
	"github.com/rs/cors"
	"gitlab.servus.at/autoradio/tank/api/v1"
	"gitlab.servus.at/autoradio/tank/importer"
	"gitlab.servus.at/autoradio/tank/store"
	"gitlab.servus.at/autoradio/tank/ui"
)

const (
	WebUIPathPrefix = "/ui/"
	WebAPIv1Prefix  = "/api/v1/"
)

func runWeb(ln net.Listener, st *store.Store, im *importer.Importer, cc *CorsConfig) error {
	r := mux.NewRouter()

	r.Handle("/", http.RedirectHandler(WebUIPathPrefix, http.StatusSeeOther))
	r.PathPrefix(WebUIPathPrefix).Handler(http.StripPrefix(WebUIPathPrefix, http.FileServer(ui.Assets)))

	v1.InstallHandler(r.PathPrefix(WebAPIv1Prefix).Subrouter(), st, im, infoLog, errLog, dbgLog)

	srv := &http.Server{
		Handler:      r,
		WriteTimeout: 12 * time.Hour,
		ReadTimeout:  12 * time.Hour,
	}

	if cc != nil {
		c := cors.New(cors.Options{
			AllowedOrigins:     cc.AllowedOrigins,
			AllowedMethods:     cc.AllowedMethods,
			AllowedHeaders:     cc.AllowedHeaders,
			ExposedHeaders:     cc.ExposedHeaders,
			MaxAge:             cc.MaxAge,
			AllowCredentials:   cc.AllowCredentials,
			OptionsPassthrough: cc.OptionsPassthrough,
			Debug:              cc.Debug,
		})
		srv.Handler = c.Handler(r)
	}

	infoLog.Printf("web: listening on %s", ln.Addr())
	return srv.Serve(ln)
}
